@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2023 - 2024 Mikhail Medvedev <e-ink-reader@yandex.ru>
+ * Copyright (C) 2023 - 2025 Mikhail Medvedev <e-ink-reader@yandex.ru>
  *
  *
  * This program is free software; you can redistribute it and/or
@@ -19,13 +19,15 @@
 #include "unistd.h"
 #include "memory"
 #include <QDebug>
+#include <QRegularExpression>
 DialogSFDP::DialogSFDP(QWidget *parent) :
     QDialog(parent),
     ui(new Ui::DialogSFDP)
 {
     ui->setupUi(this);
+    setWindowFlags(Qt::Window | Qt::WindowMinimizeButtonHint | Qt::WindowCloseButtonHint);
     setLineEditFilter();
-    r1Enable();
+    setRegStatus(1, true);
     numOfRegisters = 3; // 3-not reading, 2 - three registers, 1 - two registers, 0 - one register
 }
 
@@ -81,9 +83,9 @@ void DialogSFDP::on_pushButton_clicked()
            return;
         }
         jedecMan = sfdpBuf[0];
-        ui->lineEdit_jedec0->setText(bP(sfdpBuf[0]));
-        ui->lineEdit_jedec1->setText(bP(sfdpBuf[1]));
-        ui->lineEdit_jedec2->setText(bP(sfdpBuf[2]));
+        ui->lineEdit_jedec0->setText(bytePrint(sfdpBuf[0]));
+        ui->lineEdit_jedec1->setText(bytePrint(sfdpBuf[1]));
+        ui->lineEdit_jedec2->setText(bytePrint(sfdpBuf[2]));
 
 
         // Reading SFDP. Transfer to ch341 0x5a
@@ -134,8 +136,8 @@ void DialogSFDP::on_pushButton_clicked()
            {
               if ((sfdpBuf[manufAreaAddress] != 0xff) && (sfdpBuf[manufAreaAddress + 1] != 0xff))
               {
-                  VCCmax = bP(sfdpBuf[manufAreaAddress + 1]) + bP(sfdpBuf[manufAreaAddress]);
-                  VCCmin = bP(sfdpBuf[manufAreaAddress + 3]) + bP(sfdpBuf[manufAreaAddress + 2]);
+                  VCCmax = bytePrint(sfdpBuf[manufAreaAddress + 1]) + bytePrint(sfdpBuf[manufAreaAddress]);
+                  VCCmin = bytePrint(sfdpBuf[manufAreaAddress + 3]) + bytePrint(sfdpBuf[manufAreaAddress + 2]);
                   VCCmax.insert(1, ".");
                   VCCmin.insert(1, ".");
                   ui->lineEdit_vcc_max->setText(VCCmax);
@@ -167,13 +169,13 @@ void DialogSFDP::on_pushButton_clicked()
            if (sfdpBuf[twoAreaAddress + 0x0f] == 0xbb) speeds = speeds + "/Dual";
            if (sfdpBuf[twoAreaAddress + 0x09] == 0xeb) speeds = speeds + "/Quad";
            ui->lineEdit_speeds->setText(speeds);
-           legendPrint("00", bP(twoAreaAddress), bP(manufAreaAddress));
+           legendPrint("00", bytePrint(twoAreaAddress), bytePrint(manufAreaAddress));
            //HEXDUMP
            regData = tr("<html><head/><body><p> Hex SFDP register data:\n");
            addrTxt = tr("<html><head/><body><p>Addr:<br>");
            for (i=0; i<232;i=i+16)
            {
-               addrTxt = addrTxt + "0" + bP(i) + "><br>";
+               addrTxt = addrTxt + "0" + bytePrint(i) + "><br>";
            }
            addrTxt = addrTxt + "0F0></p></body></html>";
            ui->label_10->setText(addrTxt);
@@ -183,16 +185,16 @@ void DialogSFDP::on_pushButton_clicked()
               if (i == 0x0c) regData = regData + "<span style=\" background:#f77;\">";
               if (i == 0x0d) regData = regData + "</span>";
 
-              if ((i == 0x14) && (jedecMan == sfdpBuf[0x10])) regData = regData + "<span style=\" background:#7f7;\">";
-              if ((i == 0x15) && (jedecMan == sfdpBuf[0x10])) regData = regData + "</span>";
-              if ((i == 0x1c) && (jedecMan != sfdpBuf[0x10])) regData = regData + "<span style=\" background:#7f7;\">";
-              if ((i == 0x1d) && (jedecMan != sfdpBuf[0x10])) regData = regData + "</span>";
+              if ((i == 0x14) && (jedecMan == sfdpBuf[0x10]) && (sfdpBuf[0x06] !=0)) regData = regData + "<span style=\" background:#7f7;\">";
+              if ((i == 0x15) && (jedecMan == sfdpBuf[0x10]) && (sfdpBuf[0x06] !=0)) regData = regData + "</span>";
+              if ((i == 0x1c) && (jedecMan != sfdpBuf[0x10]) && (sfdpBuf[0x06] !=0)) regData = regData + "<span style=\" background:#7f7;\">";
+              if ((i == 0x1d) && (jedecMan != sfdpBuf[0x10]) && (sfdpBuf[0x06] !=0)) regData = regData + "</span>";
 
               if (i == twoAreaAddress) regData = regData + "<span style=\" background:#f77;\">";
               if (i == twoAreaAddress + twoAreaLen) regData = regData + "</span>";
               if (i == manufAreaAddress) regData = regData + "<span style=\" background:#7f7;\">";
               if (i == manufAreaAddress + manAreaLen) regData = regData + "</span>";
-              regData = regData + bP(sfdpBuf[i]) + " ";
+              regData = regData + bytePrint(sfdpBuf[i]) + " ";
            }
            regData = regData + "</p></body></html>";
            ui->label->setText(regData);
@@ -209,14 +211,7 @@ void DialogSFDP::on_pushButton_clicked()
            QMessageBox::about(this, tr("Error"), tr("Error reading register!"));
            return;
         }
-        ui->lineEdit_sr07->setText(QString::number(((sfdpBuf[0] & 128) >> 7)));
-        ui->lineEdit_sr06->setText(QString::number(((sfdpBuf[0] & 64) >> 6)));
-        ui->lineEdit_sr05->setText(QString::number(((sfdpBuf[0] & 32) >> 5)));
-        ui->lineEdit_sr04->setText(QString::number(((sfdpBuf[0] & 16) >> 4)));
-        ui->lineEdit_sr03->setText(QString::number(((sfdpBuf[0] & 8) >> 3)));
-        ui->lineEdit_sr02->setText(QString::number(((sfdpBuf[0] & 4) >> 2)));
-        ui->lineEdit_sr01->setText(QString::number(((sfdpBuf[0] & 2) >> 1)));
-        ui->lineEdit_sr00->setText(QString::number((sfdpBuf[0] & 1)));
+        printRegData(0, sfdpBuf[0]);
         //READING STATUS REGISTER 1
         numOfRegisters = 0;
         SPI_CONTROLLER_Chip_Select_Low();
@@ -232,17 +227,10 @@ void DialogSFDP::on_pushButton_clicked()
         if (sfdpBuf[0] != 0xff)
         {
             numOfRegisters++;
-            r1Enable();
-            ui->lineEdit_sr17->setText(QString::number(((sfdpBuf[0] & 128) >> 7)));
-            ui->lineEdit_sr16->setText(QString::number(((sfdpBuf[0] & 64) >> 6)));
-            ui->lineEdit_sr15->setText(QString::number(((sfdpBuf[0] & 32) >> 5)));
-            ui->lineEdit_sr14->setText(QString::number(((sfdpBuf[0] & 16) >> 4)));
-            ui->lineEdit_sr13->setText(QString::number(((sfdpBuf[0] & 8) >> 3)));
-            ui->lineEdit_sr12->setText(QString::number(((sfdpBuf[0] & 4) >> 2)));
-            ui->lineEdit_sr11->setText(QString::number(((sfdpBuf[0] & 2) >> 1)));
-            ui->lineEdit_sr10->setText(QString::number((sfdpBuf[0] & 1)));
+            setRegStatus(1, true);
+            printRegData(1, sfdpBuf[0]);
         }
-        else r1Disable();
+        else setRegStatus(1, false);
 
         //READING STATUS REGISTER 2
         SPI_CONTROLLER_Chip_Select_Low();
@@ -258,17 +246,10 @@ void DialogSFDP::on_pushButton_clicked()
         if (sfdpBuf[0] != 0xff)
         {
             numOfRegisters++;
-            r2Enable();
-            ui->lineEdit_sr27->setText(QString::number(((sfdpBuf[0] & 128) >> 7)));
-            ui->lineEdit_sr26->setText(QString::number(((sfdpBuf[0] & 64) >> 6)));
-            ui->lineEdit_sr25->setText(QString::number(((sfdpBuf[0] & 32) >> 5)));
-            ui->lineEdit_sr24->setText(QString::number(((sfdpBuf[0] & 16) >> 4)));
-            ui->lineEdit_sr23->setText(QString::number(((sfdpBuf[0] & 8) >> 3)));
-            ui->lineEdit_sr22->setText(QString::number(((sfdpBuf[0] & 4) >> 2)));
-            ui->lineEdit_sr21->setText(QString::number(((sfdpBuf[0] & 2) >> 1)));
-            ui->lineEdit_sr20->setText(QString::number((sfdpBuf[0] & 1)));
+            setRegStatus(2, true);
+            printRegData(2,sfdpBuf[0]);
         }
-        else r2Disable();
+        else setRegStatus(2, false);
 
         //Reading Unique ID
         SPI_CONTROLLER_Chip_Select_Low();
@@ -289,7 +270,7 @@ void DialogSFDP::on_pushButton_clicked()
         regData = "";
         for (i=0; i<idSize; i++)
         {
-            regData = regData + bP(sfdpBuf[i]);
+            regData = regData + bytePrint(sfdpBuf[i]);
         }
         ui->lineEdit_chipid->setText(regData);
 
@@ -299,17 +280,6 @@ void DialogSFDP::on_pushButton_clicked()
     else QMessageBox::about(this, tr("Error"), tr("Programmer CH341a is not connected!"));
 
 }
-QString DialogSFDP::bP(unsigned char z)
-{
-    unsigned char s;
-    s = z / 16;
-    if (s > 0x9) s = s + 0x37;
-    else s = s + 0x30;
-    z = z % 16;
-    if (z > 0x9) z = z + 0x37;
-    else z = z + 0x30;
-    return QString(s) + QString(z);
-}
 
 void DialogSFDP::on_pushButton_2_clicked()
 {
@@ -324,37 +294,28 @@ void DialogSFDP::on_pushButton_3_clicked()
     stCH341 = ch341a_spi_init();
     if (stCH341 == 0)
     {
-       uint8_t r0, r1, r2;
-
-       r0 = 0;
-       r1 = 0;
-       r2 = 0;
-       if (QString::compare(ui->lineEdit_sr07->text(), "0", Qt::CaseInsensitive)) r0 = r0 + 128;
-       if (QString::compare(ui->lineEdit_sr06->text(), "0", Qt::CaseInsensitive)) r0 = r0 +  64;
-       if (QString::compare(ui->lineEdit_sr05->text(), "0", Qt::CaseInsensitive)) r0 = r0 +  32;
-       if (QString::compare(ui->lineEdit_sr04->text(), "0", Qt::CaseInsensitive)) r0 = r0 +  16;
-       if (QString::compare(ui->lineEdit_sr03->text(), "0", Qt::CaseInsensitive)) r0 = r0 +   8;
-       if (QString::compare(ui->lineEdit_sr02->text(), "0", Qt::CaseInsensitive)) r0 = r0 +   4;
-       if (QString::compare(ui->lineEdit_sr01->text(), "0", Qt::CaseInsensitive)) r0 = r0 +   2;
-       if (QString::compare(ui->lineEdit_sr00->text(), "0", Qt::CaseInsensitive)) r0 = r0 +   1;
-
-       if (QString::compare(ui->lineEdit_sr17->text(), "0", Qt::CaseInsensitive)) r1 = r1 + 128;
-       if (QString::compare(ui->lineEdit_sr16->text(), "0", Qt::CaseInsensitive)) r1 = r1 +  64;
-       if (QString::compare(ui->lineEdit_sr15->text(), "0", Qt::CaseInsensitive)) r1 = r1 +  32;
-       if (QString::compare(ui->lineEdit_sr14->text(), "0", Qt::CaseInsensitive)) r1 = r1 +  16;
-       if (QString::compare(ui->lineEdit_sr13->text(), "0", Qt::CaseInsensitive)) r1 = r1 +   8;
-       if (QString::compare(ui->lineEdit_sr12->text(), "0", Qt::CaseInsensitive)) r1 = r1 +   4;
-       if (QString::compare(ui->lineEdit_sr11->text(), "0", Qt::CaseInsensitive)) r1 = r1 +   2;
-       if (QString::compare(ui->lineEdit_sr10->text(), "0", Qt::CaseInsensitive)) r1 = r1 +   1;
-
-       if (QString::compare(ui->lineEdit_sr27->text(), "0", Qt::CaseInsensitive)) r2 = r2 + 128;
-       if (QString::compare(ui->lineEdit_sr26->text(), "0", Qt::CaseInsensitive)) r2 = r2 +  64;
-       if (QString::compare(ui->lineEdit_sr25->text(), "0", Qt::CaseInsensitive)) r2 = r2 +  32;
-       if (QString::compare(ui->lineEdit_sr24->text(), "0", Qt::CaseInsensitive)) r2 = r2 +  16;
-       if (QString::compare(ui->lineEdit_sr23->text(), "0", Qt::CaseInsensitive)) r2 = r2 +   8;
-       if (QString::compare(ui->lineEdit_sr22->text(), "0", Qt::CaseInsensitive)) r2 = r2 +   4;
-       if (QString::compare(ui->lineEdit_sr21->text(), "0", Qt::CaseInsensitive)) r2 = r2 +   2;
-       if (QString::compare(ui->lineEdit_sr20->text(), "0", Qt::CaseInsensitive)) r2 = r2 +   1;
+       //scanning data from the form
+       uint8_t r[3], k, i;
+       int j;
+       r[0] = 0;
+       r[1] = 0;
+       r[2] = 0;
+       for (k = 0; k < 3; k++)
+       {
+           i = 128;
+           for (j = 7; j >= 0; j--)
+           {
+               QString currRegName = "lineEdit_sr" + QString::number(k) + QString::number(j);
+                       for (QLineEdit* edit : findChildren<QLineEdit*>())
+                       {
+                           if (edit->objectName() == currRegName)
+                           {
+                               if (QString::compare(edit->text(), "0", Qt::CaseInsensitive)) r[k] = r[k] + i;
+                               i = i / 2;
+                           }
+                       }
+           }
+       }
 
        //Writing status registers 0,1 for Winbond
 
@@ -370,8 +331,8 @@ void DialogSFDP::on_pushButton_3_clicked()
 
        SPI_CONTROLLER_Chip_Select_Low();
        SPI_CONTROLLER_Write_One_Byte(0x01);
-       SPI_CONTROLLER_Write_One_Byte(r0);
-       if (numOfRegisters > 0) SPI_CONTROLLER_Write_One_Byte(r1);
+       SPI_CONTROLLER_Write_One_Byte(r[0]);
+       if (numOfRegisters > 0) SPI_CONTROLLER_Write_One_Byte(r[1]);
        SPI_CONTROLLER_Chip_Select_High();
        usleep(1);
 
@@ -400,7 +361,7 @@ void DialogSFDP::on_pushButton_3_clicked()
 
            SPI_CONTROLLER_Chip_Select_Low();
            SPI_CONTROLLER_Write_One_Byte(0x01);
-           SPI_CONTROLLER_Write_One_Byte(r0);
+           SPI_CONTROLLER_Write_One_Byte(r[0]);
            SPI_CONTROLLER_Chip_Select_High();
            usleep(1);
 
@@ -426,7 +387,7 @@ void DialogSFDP::on_pushButton_3_clicked()
 
            SPI_CONTROLLER_Chip_Select_Low();
            SPI_CONTROLLER_Write_One_Byte(0x31);
-           SPI_CONTROLLER_Write_One_Byte(r1);
+           SPI_CONTROLLER_Write_One_Byte(r[1]);
            SPI_CONTROLLER_Chip_Select_High();
            usleep(1);
 
@@ -456,7 +417,7 @@ void DialogSFDP::on_pushButton_3_clicked()
 
            SPI_CONTROLLER_Chip_Select_Low();
            SPI_CONTROLLER_Write_One_Byte(0x11);
-           SPI_CONTROLLER_Write_One_Byte(r2);
+           SPI_CONTROLLER_Write_One_Byte(r[2]);
            SPI_CONTROLLER_Chip_Select_High();
            usleep(1);
 
@@ -478,100 +439,49 @@ void DialogSFDP::setLineEditFilter()
 {
     QRegExp reHex( "[0-1]{1}" );
     QRegExpValidator *validator = new QRegExpValidator(reHex, this);
-    ui->lineEdit_sr00->setValidator(validator);
-    ui->lineEdit_sr01->setValidator(validator);
-    ui->lineEdit_sr02->setValidator(validator);
-    ui->lineEdit_sr03->setValidator(validator);
-    ui->lineEdit_sr04->setValidator(validator);
-    ui->lineEdit_sr05->setValidator(validator);
-    ui->lineEdit_sr06->setValidator(validator);
-    ui->lineEdit_sr07->setValidator(validator);
+    //searching all lineEdit_srXX, XX - numbers
+    QString searchText = "lineEdit_sr\\d+";
+        QRegularExpression regex(searchText);
 
-    ui->lineEdit_sr10->setValidator(validator);
-    ui->lineEdit_sr11->setValidator(validator);
-    ui->lineEdit_sr12->setValidator(validator);
-    ui->lineEdit_sr13->setValidator(validator);
-    ui->lineEdit_sr14->setValidator(validator);
-    ui->lineEdit_sr15->setValidator(validator);
-    ui->lineEdit_sr16->setValidator(validator);
-    ui->lineEdit_sr17->setValidator(validator);
-
-    ui->lineEdit_sr20->setValidator(validator);
-    ui->lineEdit_sr21->setValidator(validator);
-    ui->lineEdit_sr22->setValidator(validator);
-    ui->lineEdit_sr23->setValidator(validator);
-    ui->lineEdit_sr24->setValidator(validator);
-    ui->lineEdit_sr25->setValidator(validator);
-    ui->lineEdit_sr26->setValidator(validator);
-    ui->lineEdit_sr27->setValidator(validator);
+            for (QLineEdit* edit : findChildren<QLineEdit*>())
+            {
+                if (regex.match(edit->objectName()).hasMatch())
+                {
+                    edit->setValidator(validator);
+                }
+            }
 }
 
-void DialogSFDP::r1Disable()
+void DialogSFDP:: setRegStatus(uint8_t regNumber, bool state)
 {
-   ui->lineEdit_sr10->setDisabled(true);
-   ui->lineEdit_sr11->setDisabled(true);
-   ui->lineEdit_sr12->setDisabled(true);
-   ui->lineEdit_sr13->setDisabled(true);
-   ui->lineEdit_sr14->setDisabled(true);
-   ui->lineEdit_sr15->setDisabled(true);
-   ui->lineEdit_sr16->setDisabled(true);
-   ui->lineEdit_sr17->setDisabled(true);
-   ui->label_11->setDisabled(true);
-   ui->lineEdit_sr10->setText("");
-   ui->lineEdit_sr11->setText("");
-   ui->lineEdit_sr12->setText("");
-   ui->lineEdit_sr13->setText("");
-   ui->lineEdit_sr14->setText("");
-   ui->lineEdit_sr15->setText("");
-   ui->lineEdit_sr16->setText("");
-   ui->lineEdit_sr17->setText("");
-}
+    QString searchText = "lineEdit_sr" + QString::number(regNumber) + "\\d+";
+    QRegularExpression regex(searchText);
 
-void DialogSFDP::r2Disable()
-{
-   ui->lineEdit_sr20->setDisabled(true);
-   ui->lineEdit_sr21->setDisabled(true);
-   ui->lineEdit_sr22->setDisabled(true);
-   ui->lineEdit_sr23->setDisabled(true);
-   ui->lineEdit_sr24->setDisabled(true);
-   ui->lineEdit_sr25->setDisabled(true);
-   ui->lineEdit_sr26->setDisabled(true);
-   ui->lineEdit_sr27->setDisabled(true);
-   ui->label_34->setDisabled(true);
-   ui->lineEdit_sr20->setText("");
-   ui->lineEdit_sr21->setText("");
-   ui->lineEdit_sr22->setText("");
-   ui->lineEdit_sr23->setText("");
-   ui->lineEdit_sr24->setText("");
-   ui->lineEdit_sr25->setText("");
-   ui->lineEdit_sr26->setText("");
-   ui->lineEdit_sr27->setText("");
-}
-
-void DialogSFDP::r1Enable()
-{
-    ui->lineEdit_sr10->setDisabled(false);
-    ui->lineEdit_sr11->setDisabled(false);
-    ui->lineEdit_sr12->setDisabled(false);
-    ui->lineEdit_sr13->setDisabled(false);
-    ui->lineEdit_sr14->setDisabled(false);
-    ui->lineEdit_sr15->setDisabled(false);
-    ui->lineEdit_sr16->setDisabled(false);
-    ui->lineEdit_sr17->setDisabled(false);
-    ui->label_11->setDisabled(false);
-}
-
-void DialogSFDP::r2Enable()
-{
-    ui->lineEdit_sr20->setDisabled(false);
-    ui->lineEdit_sr21->setDisabled(false);
-    ui->lineEdit_sr22->setDisabled(false);
-    ui->lineEdit_sr23->setDisabled(false);
-    ui->lineEdit_sr24->setDisabled(false);
-    ui->lineEdit_sr25->setDisabled(false);
-    ui->lineEdit_sr26->setDisabled(false);
-    ui->lineEdit_sr27->setDisabled(false);
-    ui->label_34->setDisabled(false);
+        for (QLineEdit* edit : findChildren<QLineEdit*>())
+        {
+            if (regex.match(edit->objectName()).hasMatch())
+            {
+                if (state == false)
+                {
+                    edit->setDisabled(true);
+                    edit->setText("");
+                }
+                else
+                {
+                    edit->setDisabled(false);
+                }
+            }
+        }
+        if (state == true)
+        {
+            if (regNumber == 1) ui->label_11->setDisabled(false);
+            if (regNumber == 2) ui->label_34->setDisabled(false);
+        }
+        else
+        {
+            if (regNumber == 1) ui->label_11->setDisabled(true);
+            if (regNumber == 2) ui->label_34->setDisabled(true);
+        }
 }
 
 void DialogSFDP::closeEvent(QCloseEvent* event)
@@ -579,3 +489,23 @@ void DialogSFDP::closeEvent(QCloseEvent* event)
     emit closeRequestHasArrived();
     QWidget::closeEvent(event);
 }
+
+void DialogSFDP::printRegData(uint8_t regNumber, uint8_t regData)
+{
+    uint8_t i;
+    int j;
+    i = 128;
+
+    for (j = 7; j >= 0; j--)
+    {
+        QString currRegName = "lineEdit_sr" + QString::number(regNumber) + QString::number(j);
+        for (QLineEdit* edit : findChildren<QLineEdit*>())
+        {
+            if (edit->objectName() == currRegName)
+            {
+                edit->setText(QString::number(((regData & i) >> j)));
+            }
+        }
+        i = i / 2;
+    }
+}
