@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2023 - 2024 Mikhail Medvedev <e-ink-reader@yandex.ru>
+ * Copyright (C) 2023 - 2025 Mikhail Medvedev <e-ink-reader@yandex.ru>
  *
  *
  * This program is free software; you can redistribute it and/or
@@ -26,6 +26,7 @@
 #include "dialogrp.h"
 #include "dialogsetaddr.h"
 #include "dialogsecurity.h"
+#include "hexutility.h"
 #include <stddef.h>
 #include <stdint.h>
 
@@ -44,7 +45,9 @@ MainWindow::MainWindow(QWidget *parent) :
  connect(timer, SIGNAL(timeout()), this, SLOT(slotTimerAlarm()));
  timer->start(2000);
 
+ ui->statusMessage->setText("");
  ui->actionStop->setDisabled(true);
+ ui->statusBar->addPermanentWidget(ui->statusMessage,1);
  ui->statusBar->addPermanentWidget(ui->lStatus,0);
  ui->statusBar->addPermanentWidget(ui->eStatus,0);
  ui->statusBar->addPermanentWidget(ui->jLabel,0);
@@ -105,6 +108,8 @@ MainWindow::MainWindow(QWidget *parent) :
  blockStartAddr = 0;
  blockLen = 0;
  currentAddr4bit = 0;
+ filled = 0;
+ numberOfReads = 0;
  cmdStarted = false;
  // connect and status check
  statusCH341 = ch341a_spi_init();
@@ -112,6 +117,9 @@ MainWindow::MainWindow(QWidget *parent) :
  chipData.reserve(256 * 1024 *1024 + 2048);
  chipData.resize(256);
  chipData.fill(char(0xff));
+ oldChipData.reserve(256 * 1024 *1024 + 2048);
+ oldChipData.resize(256);
+ oldChipData.fill(char(0xff));
  ch341a_spi_shutdown();
  QFont heFont;
  heFont = QFont("Monospace", 10);
@@ -119,10 +127,6 @@ MainWindow::MainWindow(QWidget *parent) :
  hexEdit->setGeometry(0,0,ui->frame->width(),ui->frame->height());
  hexEdit->setData(chipData);
  hexEdit->setHexCaps(true);
- defaultTextColor = ui->label->palette().color(QPalette::Text);
- hexEdit->setAsciiFontColor(defaultTextColor);
- hexEdit->setAddressFontColor(defaultTextColor);
- hexEdit->setHexFontColor(defaultTextColor);
  hexEdit->setFont(heFont);
  QStringList commandLineParams = QCoreApplication::arguments();
  QString commandLineFileName ="";
@@ -147,7 +151,8 @@ MainWindow::~MainWindow()
 
 void MainWindow::on_pushButton_clicked()
 {
-  //Reading data from chip
+  //Reading data from chip  
+  newFileName = ui->comboBox_name->currentText();
   int res = 0;
   uint32_t numBlocks, step;
   statusCH341 = ch341a_init(currentChipType, currentI2CBusSpeed);
@@ -163,6 +168,11 @@ void MainWindow::on_pushButton_clicked()
     {
        doNotDisturb();
        ch341StatusFlashing();
+       if (numberOfReads > 0)
+       {
+           oldChipData = chipData;
+           oldFileName = ui->comboBox_name->currentText();
+       }
        uint32_t addr = 0;
        uint32_t curBlock = 0;
        uint32_t j, k;
@@ -191,7 +201,7 @@ void MainWindow::on_pushButton_clicked()
        ui->progressBar->setValue(0);
        std::shared_ptr<uint8_t[]> buf(new uint8_t[step]);
        ui->pushButton->setStyleSheet(redKeyStyle);
-       ui->statusBar->showMessage(tr("Reading data from ") + ui->comboBox_name->currentText());
+       ui->statusMessage->setText(tr("Reading data from ") + ui->comboBox_name->currentText());
        for (k = 0; k < numBlocks; k++)
        {
            switch (currentChipType)
@@ -236,6 +246,7 @@ void MainWindow::on_pushButton_clicked()
             {
                QMessageBox::about(this, tr("Error"), tr("Error reading block ") + QString::number(curBlock));
                ch341a_spi_shutdown();
+               ui->pushButton->setStyleSheet(grnKeyStyle);
                doNotDisturbCancel();
                return;
             }
@@ -264,10 +275,11 @@ void MainWindow::on_pushButton_clicked()
        if (currentChipType  >0 ) QMessageBox::about(this, tr("Error"), tr("Please select the chip parameters - manufacture and chip name"));
     }
     hexEdit->setData(chipData);
-    ui->statusBar->showMessage("");
+    ui->statusMessage->setText("");
     ui->progressBar->setValue(0);
     ui->pushButton->setStyleSheet(grnKeyStyle);
-    ui->crcEdit->setText(getCRC32());
+    ui->crcEdit->setText(getCRC32(chipData));
+    newFileName = ui->comboBox_name->currentText();
   }
   else
   {
@@ -276,6 +288,8 @@ void MainWindow::on_pushButton_clicked()
   }
   ch341a_spi_shutdown();
   doNotDisturbCancel();
+  filled = 0;
+  numberOfReads++;
 }
 
 void MainWindow::on_pushButton_2_clicked()
@@ -478,26 +492,9 @@ void MainWindow::on_pushButton_2_clicked()
             }
         }
     }
-    currentChipSize = ui->comboBox_size->currentData().toUInt();
-    currentBlockSize = ui->comboBox_block->currentData().toUInt();
-    currentPageSize = ui->comboBox_page->currentData().toUInt();
-    currentAddr4bit = ui->comboBox_addr4bit->currentData().toUInt();
-    if ((currentChipSize !=0) && (currentBlockSize!=0)  && (currentChipType == 0))
-    {
-    currentNumBlocks = currentChipSize / currentBlockSize;
-    chipData.resize(static_cast<int>(currentChipSize));
-    chipData.fill(char(0xff));
-    hexEdit->setData(chipData);
-    }
-    if ((currentChipSize !=0) && (currentPageSize!=0)  && (currentChipType == 1))
-    {
-    currentNumBlocks = currentChipSize / currentPageSize;
-    chipData.resize(static_cast<int>(currentChipSize));
-    chipData.fill(char(0xff));
-    hexEdit->setData(chipData);
-    }
+
     ui->pushButton_2->setStyleSheet(grnKeyStyle);
-    ui->crcEdit->setText(getCRC32());
+    ui->crcEdit->setText(getCRC32(chipData));
     ch341a_spi_shutdown();
     timer->start();
 }
@@ -511,15 +508,21 @@ void MainWindow::on_comboBox_size_currentIndexChanged(int index)
     if ((currentChipSize !=0) && (currentBlockSize!=0) && (currentChipType == 0))
     {
         currentNumBlocks = currentChipSize / currentBlockSize;
+        preparingToCompare(1);
+        numberOfReads = 0;
         chipData.resize(static_cast<int>(currentChipSize));
         chipData.fill(char(0xff));
+        filled = 1;
         hexEdit->setData(chipData);
     }
     if ((currentChipSize !=0) && (currentPageSize!=0)  && (currentChipType > 0))
     {
     currentNumBlocks = currentChipSize / currentPageSize;
+    preparingToCompare(1);
+    numberOfReads = 0;
     chipData.resize(static_cast<int>(currentChipSize));
     chipData.fill(char(0xff));
+    filled = 1;
     hexEdit->setData(chipData);
     }
     index = index + 0;
@@ -534,21 +537,14 @@ void MainWindow::on_comboBox_page_currentIndexChanged(int index)
     if ((currentChipSize !=0) && (currentBlockSize!=0) && (currentChipType ==0))
     {
         currentNumBlocks = currentChipSize / currentBlockSize;
-        chipData.resize(static_cast<int>(currentChipSize));
-        chipData.fill(char(0xff));
-        hexEdit->setData(chipData);
     }
     if ((currentChipSize !=0) && (currentPageSize!=0)  && (currentChipType > 0))
     {
     currentNumBlocks = currentChipSize / currentPageSize;
-    chipData.resize(static_cast<int>(currentChipSize));
-    chipData.fill(char(0xff));
-    hexEdit->setData(chipData);
     }
     index = index + 0;
 }
 
-
 void MainWindow::on_actionDetect_triggered()
 {
    MainWindow::on_pushButton_2_clicked();
@@ -562,7 +558,7 @@ void MainWindow::on_actionSave_triggered()
     lastDirectory.replace(".hex", ".bin");
     lastDirectory.replace(".HEX", ".bin");
 
-    ui->statusBar->showMessage(tr("Saving file"));
+    ui->statusMessage->setText(tr("Saving file"));
     fileName = QFileDialog::getSaveFileName(this,
                                 QString(tr("Save file")),
                                 lastDirectory,
@@ -580,6 +576,7 @@ void MainWindow::on_actionSave_triggered()
     }
     file.write(hexEdit->data());
     file.close();
+    ui->statusMessage->setText("");
 }
 
 void MainWindow::on_actionErase_triggered()
@@ -594,7 +591,7 @@ void MainWindow::on_actionErase_triggered()
         QMessageBox::about(this, tr("Error"), tr("Programmer CH341a is not connected!"));
         return;
       }
-    ui->statusBar->showMessage(tr("Erasing the ") + ui->comboBox_name->currentText());
+    ui->statusMessage->setText(tr("Erasing the ") + ui->comboBox_name->currentText());
     ui->checkBox->setStyleSheet("QCheckBox{font-weight:600;}");
     ui->centralWidget->repaint();
     ui->progressBar->setRange(0, 100);
@@ -733,7 +730,7 @@ void MainWindow::on_actionErase_triggered()
     }
     doNotDisturbCancel();
     ui->checkBox->setStyleSheet("");
-    ui->statusBar->showMessage("");
+    ui->statusMessage->setText("");
     ui->progressBar->setValue(0);
     ui->centralWidget->repaint();
     ch341a_spi_shutdown();
@@ -752,7 +749,9 @@ void MainWindow::on_actionRedo_triggered()
 void MainWindow::on_actionOpen_triggered()
 {    
     QByteArray buf;
-    ui->statusBar->showMessage(tr("Opening file"));
+    ui->statusMessage->setText(tr("Opening file"));
+    if (numberOfReads == 0) oldFileName = fileName;
+    else oldFileName = ui->comboBox_name->currentText();
     if (!cmdStarted)
     {
         fileName = QFileDialog::getOpenFileName(this,
@@ -764,12 +763,12 @@ void MainWindow::on_actionOpen_triggered()
    cmdStarted = false;
 
     QFileInfo info(fileName);
-    ui->statusBar->showMessage(tr("Current file: ") + info.fileName());
     lastDirectory = info.filePath();
     // if ChipSze = 0 (Chip is not selected) IMSProg using at hexeditor only. chipsize -> hexedit.data
     // if ChipSize < FileSize - showing error message
     // if Filesize <= ChipSize - filling fileArray to hexedit.Data, the end of the array chipData remains filled 0xff
     QFile file(fileName);
+    ui->statusMessage->setText("");
     if ((info.size() > currentChipSize) && (currentChipSize != 0))
     {
       QMessageBox::about(this, tr("Error"), tr("The file size exceeds the chip size. Please select another chip or file or use `Save part` to split the file."));
@@ -780,6 +779,9 @@ void MainWindow::on_actionOpen_triggered()
 
         return;
     }
+    ui->statusMessage->setText(tr("Current file: ") + info.fileName());
+    preparingToCompare(0);
+    filled = 0;
     buf.resize(static_cast<int>(info.size()));
     buf = file.readAll();
     if (currentChipSize == 0)
@@ -791,20 +793,22 @@ void MainWindow::on_actionOpen_triggered()
     hexEdit->setData(chipData);
 
     file.close();
-    //ui->statusBar->showMessage("");
-    ui->crcEdit->setText(getCRC32());
+
+    ui->crcEdit->setText(getCRC32(chipData));
 }
 
 void MainWindow::on_actionExtract_from_ASUS_CAP_triggered()
 {
     QByteArray buf;
-    ui->statusBar->showMessage(tr("Opening file"));
+    ui->statusMessage->setText(tr("Opening file"));
+    if (numberOfReads == 0) oldFileName = fileName;
+    else oldFileName = ui->comboBox_name->currentText();
     fileName = QFileDialog::getOpenFileName(this,
                                 QString(tr("Open file")),
                                 lastDirectory,
                                 "ASUS Data Images (*.cap *.CAP);;All files (*.*)");
     QFileInfo info(fileName);
-    ui->statusBar->showMessage(tr("Current file: ") + info.fileName());
+    ui->statusMessage->setText("");
     lastDirectory = info.filePath();
     if ((info.size() - 0x800 > currentChipSize) && (currentChipSize != 0))
     {
@@ -818,6 +822,9 @@ void MainWindow::on_actionExtract_from_ASUS_CAP_triggered()
 
         return;
     }
+    ui->statusMessage->setText(tr("Current file: ") + info.fileName());
+    preparingToCompare(0);
+    filled = 0;
     buf.resize(int(info.size()));
     buf = file.readAll();
     file.close();
@@ -829,7 +836,7 @@ void MainWindow::on_actionExtract_from_ASUS_CAP_triggered()
     }
     chipData.replace(0, static_cast<int>(info.size()) - 0x800, buf);
     hexEdit->setData(chipData);
-    ui->crcEdit->setText(getCRC32());
+    ui->crcEdit->setText(getCRC32(chipData));
 }
 
 void MainWindow::on_actionWrite_triggered()
@@ -872,7 +879,7 @@ void MainWindow::on_actionWrite_triggered()
     uint32_t addr = 0;
     uint32_t curBlock = 0;    
     uint32_t j, k;
-    ui->statusBar->showMessage(tr("Writing data to ") + ui->comboBox_name->currentText());
+    ui->statusMessage->setText(tr("Writing data to ") + ui->comboBox_name->currentText());
     //progerssbar settings
     ui->progressBar->setRange(0, static_cast<int>(numBlocks));
     ui->checkBox_2->setStyleSheet("QCheckBox{font-weight:800;}");
@@ -950,7 +957,7 @@ void MainWindow::on_actionWrite_triggered()
     doNotDisturbCancel();
     ui->progressBar->setValue(0);
     ui->checkBox_2->setStyleSheet("");
-    ui->statusBar->showMessage("");    
+    ui->statusMessage->setText("");    
     }
     else
     {
@@ -995,7 +1002,7 @@ void MainWindow::on_comboBox_man_currentIndexChanged(int index)
         ui->comboBox_block->setCurrentIndex(0);
         ui->comboBox_size->setCurrentIndex(0);
         ui->comboBox_addr4bit->setCurrentIndex(0);
-        ui->statusBar->showMessage("");
+        ui->statusMessage->setText("");
    }
  index = index + 0;
 }
@@ -1003,6 +1010,7 @@ void MainWindow::on_comboBox_man_currentIndexChanged(int index)
 void MainWindow::on_comboBox_name_currentIndexChanged(const QString &arg1)
 {
     int i, index;
+    oldFileName = fileName;
     QString manName = ui->comboBox_man->currentText();
     if (arg1.compare("") !=0)
     {
@@ -1043,12 +1051,14 @@ void MainWindow::on_comboBox_name_currentIndexChanged(const QString &arg1)
        currentBlockSize = ui->comboBox_block->currentData().toUInt();
        currentPageSize = ui->comboBox_page->currentData().toUInt();
        currentAddr4bit = ui->comboBox_addr4bit->currentData().toUInt();
+       preparingToCompare(1);
 
        if ((currentChipSize !=0) && (currentBlockSize!=0) && (currentChipType == 0))
        {
            currentNumBlocks = currentChipSize / currentBlockSize;
            chipData.resize(static_cast<int>(currentChipSize));
            chipData.fill(char(0xff));
+           filled = 1;
            hexEdit->setData(chipData);
        }
        if ((currentChipSize !=0) && (currentPageSize!=0)  && (currentChipType > 0))
@@ -1056,6 +1066,7 @@ void MainWindow::on_comboBox_name_currentIndexChanged(const QString &arg1)
            currentNumBlocks = currentChipSize / currentPageSize;
            chipData.resize(static_cast<int>(currentChipSize));
            chipData.fill(char(0xff));
+           filled = 1;
            hexEdit->setData(chipData);
        }
 
@@ -1109,7 +1120,7 @@ void MainWindow::on_actionVerify_triggered()
                std::shared_ptr<uint8_t[]> buf(new uint8_t[step]);
                chipData = hexEdit->data();
                ui->checkBox_3->setStyleSheet("QCheckBox{font-weight:800;}");
-               ui->statusBar->showMessage(tr("Veryfing data from ") + ui->comboBox_name->currentText());
+               ui->statusMessage->setText(tr("Veryfing data from ") + ui->comboBox_name->currentText());
                for (k = 0; k < numBlocks; k++)
                {
                    switch (currentChipType)
@@ -1156,6 +1167,7 @@ void MainWindow::on_actionVerify_triggered()
                     {
                         QMessageBox::about(this, tr("Error"), tr("Error reading block ") + QString::number(curBlock));
                         ch341a_spi_shutdown();
+                        ui->pushButton->setStyleSheet(grnKeyStyle);
                         doNotDisturbCancel();
                         return;
                     }
@@ -1164,8 +1176,8 @@ void MainWindow::on_actionVerify_triggered()
                       if (chipData[addr + j] != char(buf[addr + j - k * step]))
                           {
                             //error compare
-                            QMessageBox::about(this, tr("Error"), tr("Error comparing data!\nAddress:   ") + hexiAddr(addr + j) + tr("\nBuffer: ") + bytePrint( static_cast<unsigned char>(chipData[addr + j])) + tr("    Chip: ") + bytePrint(buf[addr + j - k * currentBlockSize]));
-                            ui->statusBar->showMessage("");
+                            QMessageBox::about(this, tr("Error"), tr("Error comparing data!\nAddress:   ") + hexiAddr(addr + j) + tr("\nBuffer: ") + bytePrint( static_cast<unsigned char>(chipData[addr + j])) + tr("    Chip: ") + bytePrint(buf[addr + j - k * step]));
+                            ui->statusMessage->setText("");
                             ui->checkBox_3->setStyleSheet("");
                             ch341a_spi_shutdown();
                             doNotDisturbCancel();
@@ -1193,10 +1205,10 @@ void MainWindow::on_actionVerify_triggered()
 
              }
              doNotDisturbCancel();
-             ui->statusBar->showMessage("");
+             ui->statusMessage->setText("");
              ui->progressBar->setValue(0);
              ui->checkBox_3->setStyleSheet("");
-             ui->crcEdit->setText(getCRC32());
+             ui->crcEdit->setText(getCRC32(chipData));
              ch341a_spi_shutdown();
              QMessageBox::about(this, tr("Ok!"), tr("The operation was successful!"));
     }
@@ -1244,7 +1256,7 @@ void MainWindow::receiveAddr(QString addressData)
     {
         block[ee] = chipData[ee + blockStartAddr];
     }
-    ui->statusBar->showMessage(tr("Saving block"));
+    ui->statusMessage->setText(tr("Saving block"));
     fileName = QFileDialog::getSaveFileName(this,
                                 QString(tr("Save block")),
                                 lastDirectory,
@@ -1261,6 +1273,7 @@ void MainWindow::receiveAddr(QString addressData)
     }
     file.write(block);
     file.close();
+    ui->statusMessage->setText("");
 }
 
 void MainWindow::receiveAddr2(QString addressData)
@@ -1270,12 +1283,12 @@ void MainWindow::receiveAddr2(QString addressData)
     blockStartAddr = 0;
     blockLen = 0;
     blockStartAddr = hexToInt(addressData);
-    ui->statusBar->showMessage(tr("Opening block"));
+    ui->statusMessage->setText(tr("Opening block"));
     fileName = QFileDialog::getOpenFileName(this,
                                 QString(tr("Open block")),
                                 lastDirectory,
                                 "Data Images (*.bin *.BIN);;All files (*.*)");
-    ui->statusBar->showMessage(tr("Current file: ") + fileName);
+    ui->statusMessage->setText(tr("Current file: ") + fileName);
     QFileInfo info(fileName);
     lastDirectory = info.filePath();
     QFile file(fileName);
@@ -1298,7 +1311,7 @@ void MainWindow::receiveAddr2(QString addressData)
     }
     hexEdit->setData(chipData);
     file.close();
-    ui->statusBar->showMessage("");
+    ui->statusMessage->setText("");
 }
 
 void MainWindow::on_actionSave_Part_triggered()
@@ -1324,6 +1337,7 @@ void MainWindow::on_actionFind_Replace_triggered()
     SearchDialog* searchDialog = new SearchDialog(hexEdit);
     searchDialog->show();
 }
+
 void MainWindow::ch341StatusFlashing()
 {
     if (statusCH341 == 0)
@@ -1382,7 +1396,7 @@ void MainWindow::on_comboBox_type_currentIndexChanged(int index)
           ui->comboBox_size->addItem("32 K", 32 * 1024);
           ui->comboBox_size->addItem("64 K", 64 * 1024);
           ui->comboBox_size->addItem("128 K", 128 * 1024);
-          ui->comboBox_size->addItem("128 K", 256 * 1024);
+          ui->comboBox_size->addItem("256 K", 256 * 1024);
        break;
        case 2:
           //MicroWire
@@ -1432,7 +1446,7 @@ void MainWindow::on_comboBox_type_currentIndexChanged(int index)
                     if (( index2 == -1 ) && (chips[i].chipTypeHex == currentChipType)) ui->comboBox_man->addItem(chips[i].chipManuf);
     }
      ui->comboBox_man->setCurrentIndex(0);
-     ui->statusBar->showMessage("");
+     ui->statusMessage->setText("");
      ui->comboBox_man->setCurrentIndex(0);
      ui->comboBox_vcc->setCurrentIndex(0);
      ui->comboBox_name->setCurrentIndex(0);
@@ -1508,10 +1522,11 @@ void MainWindow::on_actionAbout_triggered()
     DialogAbout* aboutDialog = new DialogAbout();
     aboutDialog->show();
 }
+
 void MainWindow::on_actionChecksum_calculate_triggered()
 {
    //Refreshing CRC32
-    ui->crcEdit->setText(getCRC32());
+    ui->crcEdit->setText(getCRC32(chipData));
 }
 
 void MainWindow::on_actionEdit_chips_Database_triggered()
@@ -1564,6 +1579,7 @@ void MainWindow::doNotDisturb()
    ui->actionRedo->setDisabled(true);
    ui->actionChecksum_calculate->setDisabled(true);
    ui->actionGoto_address->setDisabled(true);
+   ui->actionCompare_files->setDisabled(true);
    ui->actionChip_info->setDisabled(true);
    ui->actionSecurity_registers->setDisabled(true);
    ui->actionStop->setDisabled(false);
@@ -1585,6 +1601,7 @@ void MainWindow::doNotDisturb()
    hexEdit->blockSignals(true);
    timer->stop();
 }
+
 void MainWindow::doNotDisturbCancel()
 {
       if (currentChipType == 0) ui->actionDetect->setDisabled(false);
@@ -1606,6 +1623,7 @@ void MainWindow::doNotDisturbCancel()
       ui->actionRedo->setDisabled(false);
       ui->actionChecksum_calculate->setDisabled(false);
       ui->actionGoto_address->setDisabled(false);
+      ui->actionCompare_files->setDisabled(false);
       if ((currentChipType == 0) || (currentChipType > 2)) ui->actionChip_info->setDisabled(false);
       if (currentChipType == 0) ui->actionSecurity_registers->setDisabled(false);
       ui->actionStop->setDisabled(true);
@@ -1627,6 +1645,7 @@ void MainWindow::doNotDisturbCancel()
       hexEdit->blockSignals(false);
       timer->start();
 }
+
 void MainWindow::on_actionStop_triggered()
 {
   //ch341a_spi_shutdown();
@@ -1637,9 +1656,10 @@ void MainWindow::on_actionStop_triggered()
   ui->checkBox_2->setStyleSheet("");
   ui->checkBox_3->setStyleSheet("");
   ui->pushButton_3->setStyleSheet(grnKeyStyle);
-  ui->statusBar->showMessage("");
+  ui->statusMessage->setText("");
   return;
 }
+
 void MainWindow::on_pushButton_4_clicked()
 {
     //info form showing
@@ -1680,7 +1700,7 @@ void MainWindow::progInit()
     QString datFileNameReserve = "/usr/share/imsprog/IMSProg.Dat";
     QString currentDatFilePath = "";
     //opening chip database file
-    ui->statusBar->showMessage(tr("Opening DAT file"));
+    ui->statusMessage->setText(tr("Opening DAT file"));
 
     if (QFileInfo(datFileNameMain).exists()) currentDatFilePath = datFileNameMain;
     else if (QFileInfo(datFileNameReserve).exists()) currentDatFilePath = datFileNameReserve;
@@ -1695,7 +1715,7 @@ void MainWindow::progInit()
     dataChips = datfile.readAll();
     datfile.close();
     //parsing dat file
-    ui->statusBar->showMessage(tr("Parsing DAT file"));
+    ui->statusMessage->setText(tr("Parsing DAT file"));
     //parsing qbytearray
     char txtBuf[0x30];
     int i, j, recNo, dataPoz, dataSize, delay;
@@ -1795,7 +1815,7 @@ void MainWindow::progInit()
                     if ((index2 == -1) && (chips[i].chipTypeHex ==0)) ui->comboBox_man->addItem(chips[i].chipManuf);
     }
      ui->comboBox_man->setCurrentIndex(0);
-     ui->statusBar->showMessage("");
+     ui->statusMessage->setText("");
      currentChipType = 0;
      ui->comboBox_type->setCurrentIndex(0);
 }
@@ -1823,10 +1843,9 @@ void MainWindow::closeSR()
    timer->start();
 }
 
-//HExEditor --> goto address
 void MainWindow::on_actionGoto_address_triggered()
 {
-
+    //HExEditor --> goto address
     DialogSetAddr* gotoAddrDialog = new DialogSetAddr();
     gotoAddrDialog->show();
     connect(gotoAddrDialog, SIGNAL(sendAddr3(qint64)), this, SLOT(receiveAddr3(qint64)));
@@ -1859,149 +1878,6 @@ void MainWindow::on_actionSecurity_registers_triggered()
 
 }
 
-//*****************************************************
-//       HEX ULTLITY by Mikhail Medvedev
-//*****************************************************
-uint32_t MainWindow::hexToInt(QString str)
-{
-    unsigned char c;
-    uint32_t len = static_cast<uint32_t>(str.length());
-    QByteArray bstr = str.toLocal8Bit();
-    if ((len > 0) && (len < 8))
-    {
-        uint32_t i, j = 1;
-        uint32_t  addr = 0;
-        for (i = len; i >0; i--)
-        {
-           c = static_cast<unsigned char>(bstr[i-1]);
-           if ((c >= 0x30) && (c <=0x39)) addr =  addr + (c - 0x30) * j;
-           if ((c >= 0x41) && (c <= 0x46)) addr = addr + (c - 0x37) * j;
-           if ((c >= 0x61) && (c <= 0x66)) addr = addr + (c - 0x57) * j;
-        j = j * 16;
-        }
-        return addr;
-    }
-    else return 0;
-}
-
-QString MainWindow::hexiAddr(uint32_t add)
-{
- QString rez = "";
- uint8_t A,B,C,D;
- D = 0xFF & add;
- add = add >> 8;
- C = 0xFF & add;
- add = add >> 8;
- B = 0xFF & add;
- add = add >> 8;
- A = 0xFF & add;
- rez = bytePrint(A) + bytePrint(B) + bytePrint(C) + bytePrint(D);
- return rez;
-}
-
-QString MainWindow::bytePrint(unsigned char z)
-{
-    unsigned char s;
-    s = z / 16;
-    if (s > 0x9) s = s + 0x37;
-    else s = s + 0x30;
-    z = z % 16;
-    if (z > 0x9) z = z + 0x37;
-    else z = z + 0x30;
-    return QString(s) + QString(z);
-}
-
-QString MainWindow::sizeConvert(int a)
-{
-    QString rez;
-    rez = "0";
-    if (a < 1024) rez = QString::number(a) + " B";
-    else if ((a < 1024 * 1024)) rez = QString::number(a/1024) + " K";
-    else rez = QString::number(a/1024/1024) + " M";
-    return rez;
-}
-
-QString MainWindow::getCRC32()
-{
-    chipData = hexEdit->data();
-    uint32_t size, i;
-    uint_least32_t crc = 0xFFFFFFFF;
-    const uint_least32_t Crc32Table[256] = {
-        0x00000000, 0x77073096, 0xEE0E612C, 0x990951BA,
-        0x076DC419, 0x706AF48F, 0xE963A535, 0x9E6495A3,
-        0x0EDB8832, 0x79DCB8A4, 0xE0D5E91E, 0x97D2D988,
-        0x09B64C2B, 0x7EB17CBD, 0xE7B82D07, 0x90BF1D91,
-        0x1DB71064, 0x6AB020F2, 0xF3B97148, 0x84BE41DE,
-        0x1ADAD47D, 0x6DDDE4EB, 0xF4D4B551, 0x83D385C7,
-        0x136C9856, 0x646BA8C0, 0xFD62F97A, 0x8A65C9EC,
-        0x14015C4F, 0x63066CD9, 0xFA0F3D63, 0x8D080DF5,
-        0x3B6E20C8, 0x4C69105E, 0xD56041E4, 0xA2677172,
-        0x3C03E4D1, 0x4B04D447, 0xD20D85FD, 0xA50AB56B,
-        0x35B5A8FA, 0x42B2986C, 0xDBBBC9D6, 0xACBCF940,
-        0x32D86CE3, 0x45DF5C75, 0xDCD60DCF, 0xABD13D59,
-        0x26D930AC, 0x51DE003A, 0xC8D75180, 0xBFD06116,
-        0x21B4F4B5, 0x56B3C423, 0xCFBA9599, 0xB8BDA50F,
-        0x2802B89E, 0x5F058808, 0xC60CD9B2, 0xB10BE924,
-        0x2F6F7C87, 0x58684C11, 0xC1611DAB, 0xB6662D3D,
-        0x76DC4190, 0x01DB7106, 0x98D220BC, 0xEFD5102A,
-        0x71B18589, 0x06B6B51F, 0x9FBFE4A5, 0xE8B8D433,
-        0x7807C9A2, 0x0F00F934, 0x9609A88E, 0xE10E9818,
-        0x7F6A0DBB, 0x086D3D2D, 0x91646C97, 0xE6635C01,
-        0x6B6B51F4, 0x1C6C6162, 0x856530D8, 0xF262004E,
-        0x6C0695ED, 0x1B01A57B, 0x8208F4C1, 0xF50FC457,
-        0x65B0D9C6, 0x12B7E950, 0x8BBEB8EA, 0xFCB9887C,
-        0x62DD1DDF, 0x15DA2D49, 0x8CD37CF3, 0xFBD44C65,
-        0x4DB26158, 0x3AB551CE, 0xA3BC0074, 0xD4BB30E2,
-        0x4ADFA541, 0x3DD895D7, 0xA4D1C46D, 0xD3D6F4FB,
-        0x4369E96A, 0x346ED9FC, 0xAD678846, 0xDA60B8D0,
-        0x44042D73, 0x33031DE5, 0xAA0A4C5F, 0xDD0D7CC9,
-        0x5005713C, 0x270241AA, 0xBE0B1010, 0xC90C2086,
-        0x5768B525, 0x206F85B3, 0xB966D409, 0xCE61E49F,
-        0x5EDEF90E, 0x29D9C998, 0xB0D09822, 0xC7D7A8B4,
-        0x59B33D17, 0x2EB40D81, 0xB7BD5C3B, 0xC0BA6CAD,
-        0xEDB88320, 0x9ABFB3B6, 0x03B6E20C, 0x74B1D29A,
-        0xEAD54739, 0x9DD277AF, 0x04DB2615, 0x73DC1683,
-        0xE3630B12, 0x94643B84, 0x0D6D6A3E, 0x7A6A5AA8,
-        0xE40ECF0B, 0x9309FF9D, 0x0A00AE27, 0x7D079EB1,
-        0xF00F9344, 0x8708A3D2, 0x1E01F268, 0x6906C2FE,
-        0xF762575D, 0x806567CB, 0x196C3671, 0x6E6B06E7,
-        0xFED41B76, 0x89D32BE0, 0x10DA7A5A, 0x67DD4ACC,
-        0xF9B9DF6F, 0x8EBEEFF9, 0x17B7BE43, 0x60B08ED5,
-        0xD6D6A3E8, 0xA1D1937E, 0x38D8C2C4, 0x4FDFF252,
-        0xD1BB67F1, 0xA6BC5767, 0x3FB506DD, 0x48B2364B,
-        0xD80D2BDA, 0xAF0A1B4C, 0x36034AF6, 0x41047A60,
-        0xDF60EFC3, 0xA867DF55, 0x316E8EEF, 0x4669BE79,
-        0xCB61B38C, 0xBC66831A, 0x256FD2A0, 0x5268E236,
-        0xCC0C7795, 0xBB0B4703, 0x220216B9, 0x5505262F,
-        0xC5BA3BBE, 0xB2BD0B28, 0x2BB45A92, 0x5CB36A04,
-        0xC2D7FFA7, 0xB5D0CF31, 0x2CD99E8B, 0x5BDEAE1D,
-        0x9B64C2B0, 0xEC63F226, 0x756AA39C, 0x026D930A,
-        0x9C0906A9, 0xEB0E363F, 0x72076785, 0x05005713,
-        0x95BF4A82, 0xE2B87A14, 0x7BB12BAE, 0x0CB61B38,
-        0x92D28E9B, 0xE5D5BE0D, 0x7CDCEFB7, 0x0BDBDF21,
-        0x86D3D2D4, 0xF1D4E242, 0x68DDB3F8, 0x1FDA836E,
-        0x81BE16CD, 0xF6B9265B, 0x6FB077E1, 0x18B74777,
-        0x88085AE6, 0xFF0F6A70, 0x66063BCA, 0x11010B5C,
-        0x8F659EFF, 0xF862AE69, 0x616BFFD3, 0x166CCF45,
-        0xA00AE278, 0xD70DD2EE, 0x4E048354, 0x3903B3C2,
-        0xA7672661, 0xD06016F7, 0x4969474D, 0x3E6E77DB,
-        0xAED16A4A, 0xD9D65ADC, 0x40DF0B66, 0x37D83BF0,
-        0xA9BCAE53, 0xDEBB9EC5, 0x47B2CF7F, 0x30B5FFE9,
-        0xBDBDF21C, 0xCABAC28A, 0x53B39330, 0x24B4A3A6,
-        0xBAD03605, 0xCDD70693, 0x54DE5729, 0x23D967BF,
-        0xB3667A2E, 0xC4614AB8, 0x5D681B02, 0x2A6F2B94,
-        0xB40BBE37, 0xC30C8EA1, 0x5A05DF1B, 0x2D02EF8D
-    };
-    size = static_cast<uint32_t>(chipData.length());
-    i = 0;
-    while (size--)
-            {
-              crc = (crc >> 8) ^ Crc32Table[(crc ^ static_cast<uint_least32_t>(chipData[i])) & 0xFF];
-              i++;
-            }
-        return hexiAddr(crc ^ 0xFFFFFFFF);
-}
-
 void MainWindow::on_actionExport_to_Intel_HEX_triggered()
 {
     int addr = 0, hi_addr =0;
@@ -2012,15 +1888,16 @@ void MainWindow::on_actionExport_to_Intel_HEX_triggered()
          int ostatok = 0;
          ui->progressBar->setRange(0,chipData.size());
          lastDirectory.replace(".bin", ".hex");
-         ui->statusBar->showMessage(tr("Saving file"));
+         ui->statusMessage->setText(tr("Saving file"));
          fileName = QFileDialog::getSaveFileName(this,
                                      QString(tr("Save file")),
                                      lastDirectory,
                                      "Intel HEX Images (*.hex *.HEX);;All files (*.*)");         
          QFileInfo info(fileName);
+         ui->statusMessage->setText("");
          lastDirectory = info.filePath();
          if (QString::compare(info.suffix(), "hex", Qt::CaseInsensitive)) fileName = fileName + ".hex";
-         ui->statusBar->showMessage(tr("Current file: ") + info.fileName());
+         ui->statusMessage->setText(tr("Current file: ") + info.fileName());
          QFile file(fileName);
          QTextStream stream(&file);
          if (!file.open(QIODevice::ReadWrite | QIODevice::Truncate | QIODevice::Text))
@@ -2082,21 +1959,24 @@ void MainWindow::on_actionImport_from_Intel_HEX_triggered()
     unsigned char currByte;
     uint8_t counter, checkSUM;
     QString currStr ="", strVal = "";
-    ui->statusBar->showMessage(tr("Opening file"));
+    ui->statusMessage->setText(tr("Opening file"));
+    if (numberOfReads == 0) oldFileName = fileName;
+    else oldFileName = ui->comboBox_name->currentText();
     fileName = QFileDialog::getOpenFileName(this,
                                 QString(tr("Open file")),
                                 lastDirectory,
                                 "Intel HEX Images (*.hex *.HEX);;All files (*.*)");
     QFileInfo info(fileName);
-    ui->statusBar->showMessage(tr("Current file: ") + info.fileName());
-    lastDirectory = info.filePath();
     QFile file(fileName);
 
     if (!file.open(QIODevice::ReadOnly))
     {
-
         return;
     }
+    ui->statusMessage->setText(tr("Current file: ") + info.fileName());
+    lastDirectory = info.filePath();
+    preparingToCompare(0);
+    filled = 0;
     hi_addr = 0;
     lo_addr = 0;
     ui->progressBar->setRange(0, chipSize);
@@ -2112,7 +1992,7 @@ void MainWindow::on_actionImport_from_Intel_HEX_triggered()
             fileName.clear();
             hexEdit->setData(chipData);
             ui->progressBar->setValue(0);
-            ui->crcEdit->setText(getCRC32());
+            ui->crcEdit->setText(getCRC32(chipData));
             return;
         }
         strVal = currStr.mid(1,2); //Length of data in current string
@@ -2144,7 +2024,7 @@ void MainWindow::on_actionImport_from_Intel_HEX_triggered()
                     fileName.clear();
                     hexEdit->setData(chipData);
                     ui->progressBar->setValue(0);
-                    ui->crcEdit->setText(getCRC32());
+                    ui->crcEdit->setText(getCRC32(chipData));
                     return;
                 }
 
@@ -2162,7 +2042,7 @@ void MainWindow::on_actionImport_from_Intel_HEX_triggered()
                     fileName.clear();
                     hexEdit->setData(chipData);
                     ui->progressBar->setValue(0);
-                    ui->crcEdit->setText(getCRC32());
+                    ui->crcEdit->setText(getCRC32(chipData));
                     return;
                 }
         }
@@ -2188,8 +2068,90 @@ void MainWindow::on_actionImport_from_Intel_HEX_triggered()
     fileName.clear();
     hexEdit->setData(chipData);
     ui->progressBar->setValue(0);
-    ui->crcEdit->setText(getCRC32());
+    ui->crcEdit->setText(getCRC32(chipData));
 }
 
+void MainWindow::on_actionFill_test_image_triggered()
+{
+    int fileSize, addrSize, txtSize, i, j, curPos, hiDigit;
+    char k;
+    fileSize = chipData.size();
+    ui->progressBar->setValue(0);
+    ui->progressBar->setRange(0, fileSize);
+    addrSize = 0;
+    j = fileSize;
+    hiDigit = 1;
+    while (j > 1)
+    {
+        j = j / 16;
+        addrSize ++;
+        hiDigit = hiDigit * 16;
+    }
+    char digits[16];
+    txtSize = 16 - addrSize - 4;
+    curPos = 0;
+    chipData.resize(0);
+           k = 0x40;
+           while (curPos < fileSize)
+           {
+               //String
+              chipData.append('<');
+              chipData.append('0');
+              chipData.append('x');
 
 
+              //calculate digits
+              i = hiDigit / 16;
+              for (j=addrSize - 1; j>=0; j--)
+              {
+                  digits[j] = (curPos / i) % 16;
+                  i = i / 16;
+              }
+              for (j = addrSize -1; j >=0; j--)
+              {
+                 if (digits[j] < 10) digits[j] = digits[j] + 0x30;
+                 else digits[j] = digits[j] + 0x37;
+                 chipData.append(digits[j]);
+              }
+
+              chipData.append('>');
+              for (i = 0; i < txtSize; i++)
+              {
+                  chipData.append(k);
+                  k ++;
+                  if (k > 0x7e) k = 0x40;
+              }
+              curPos = curPos + 16;
+              if (curPos % 512 == 0) ui->progressBar->setValue(curPos);
+           }
+    hexEdit->setData(chipData);
+    ui->crcEdit->setText(getCRC32(chipData));
+    ui->progressBar->setValue(0);
+}
+
+void MainWindow::on_actionCompare_files_triggered()
+{
+    DialogCompare* compDialog = new DialogCompare();
+    compDialog->show();
+    compDialog->showArrays(&chipData, &oldChipData, &newFileName, &oldFileName);
+}
+
+void MainWindow::preparingToCompare(bool type)
+{
+    //For comparing function
+    // type = 0 - file reading
+    // type = 1 - chip reading
+    if (filled == 0) oldChipData = hexEdit->data();
+    if (type == 0)
+    {
+        if (numberOfReads > 0)
+        {
+           numberOfReads = 0;
+        }
+        newFileName = fileName;
+    }
+    else
+    {
+        newFileName = ui->comboBox_name->currentText();
+    }
+}
