@@ -28,7 +28,9 @@
 #include "dialoginfo.h"
 #include "dialogsfdp.h"
 #include "dialogsr.h"
+#include "dialogcompare.h"
 #include "searchdialog.h"
+#include "hexutility.h"
 extern "C" {
 #include "bitbang_microwire.h"
 #include "ch341a_gpio.h"
@@ -108,22 +110,24 @@ private slots:
     void slotTimerAlarm();
     void on_actionGoto_address_triggered();
     void on_comboBox_i2cSpeed_currentIndexChanged(int index);
-
     void on_actionSecurity_registers_triggered();
+    void on_actionFill_test_image_triggered();
+    void preparingToCompare(bool type);
+    void on_actionCompare_files_triggered();
 
 private:
     Ui::MainWindow *ui;
-    QColor defaultTextColor;
     QString grnKeyStyle, redKeyStyle;
     QString lastDirectory;
     int statusCH341;
-    QByteArray chipData;
+    QByteArray chipData, oldChipData;
     uint32_t currentChipSize, currentNumBlocks, currentBlockSize, currentPageSize;
     uint8_t currentAlgorithm, currentChipType, currentI2CBusSpeed;
     unsigned int currentAddr4bit;
     bool isHalted;
+    bool filled;
+    uint8_t numberOfReads;
     QTimer *timer;
-    QString bytePrint(unsigned char z);
     QVector <QString> chType = {"SPI_FLASH","25_EEPROM","93_EEPROM","24_EEPROM","95_EEPROM"};
     struct chip_data {
         QString chipManuf;
@@ -144,13 +148,9 @@ private:
     };
     chip_data chips[2000];
     int max_rec;
-    QString fileName;
+    QString fileName, oldFileName, newFileName;
     bool cmdStarted;
     QHexEdit *hexEdit;
-    QString sizeConvert(int a);
-    QString hexiAddr(uint32_t a);
-    uint32_t hexToInt(QString str);
-    QString getCRC32();
     void ch341StatusFlashing();
     QByteArray block;
     uint32_t blockStartAddr, blockLen;
